
"""
GLOBAL LIST OF ALTERNATES AND SYNONYMS
"""

# Missing value name appears as many variations. Try to account for all of them
ALTS_MV = ['missing value', 'missing values', 'missingvalue', 'missingvalues', 'missing_values',
                    'missing variables', 'missing_variables', 'missingvariables']