Search.setIndex({envversion:47,filenames:["index"],objects:{},objnames:{},objtypes:{},terms:{content:0,index:0,modul:0,page:0,search:0},titles:["Welcome to LiPD&#8217;s documentation!"],titleterms:{document:0,indic:0,lipd:0,tabl:0,welcom:0}})